
package org.omegahat.DataStructures.Data;

import org.omegahat.Environment.Language.Function;

import org.omegahat.Environment.Tools.DataScanner.ObjectReader;

import org.omegahat.Environment.Parser.Parse.List;

import org.omegahat.Environment.Interpreter.Evaluator;


/**
 Extension of a {@link DataList DataList} 
that allows the user specify an interpreted Omegahat
function that is used to process each record as it 
is supplied to the 
{@link org.omegahat.Environment.Tools.DataScanner.RecordStreamListener 
    RecordStreamListener} method
({@link org.omegahat.Environment.Tools.DataScanner.RecordStreamListener#newRecord(java.lang.Object,org.omegahat.Environment.Tools.DataScanner.ObjectReader source) newRecord()}
*/
public class FilterDataList extends DataList {

public FilterDataList(DataList source) {
}

public FilterDataList(DataList source, Function f) {
  source(source);
  Function(f);
  apply(Function(),evaluator(), source());
}




public FilterDataList(ObjectReader reader, Function function) {
  super(reader);
  Function(function);
}

public FilterDataList(ObjectReader reader, Function function, org.omegahat.Environment.Interpreter.Evaluator evaluator) {
  super(reader);
  Function(function);
  evaluator(evaluator);
}

protected org.omegahat.Environment.Language.Function function;

public DataList source;

 protected org.omegahat.Environment.Interpreter.Evaluator evaluator;


public Object processElement(Object obj) {
 List list = createArgumentList(obj, Function());
 Object val  = null;
 try {
   val = Function().eval(list, evaluator());
 } catch(Throwable e) {
   evaluator.warning("Error filtering object " + size() + " " + obj);
   e.printStackTrace();
 }

return(val);
}

public Function Function() {
  return(function);
}

public Function Function(Function f) {
  function = f;
  return(Function());
}

public DataList source() {
  return(source);
}

public DataList source(DataList src) {
  source = src;
  return(source());
}

public Evaluator evaluator() {
 if(evaluator == null) {
  try {
    evaluator(org.omegahat.Environment.System.Globals.manager().evaluator()); 
  } catch(ClassNotFoundException ex) {
    evaluator(new org.omegahat.Environment.Interpreter.BasicEvaluator());
  }
 }
  return(evaluator);
}

public Evaluator evaluator(Evaluator eval) {
  evaluator = eval;
  return(evaluator());
}




}

/*
Copyright (c) 1998, 1999 The Omega Project for Statistical Computing.
     All rights reserved.

     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions are
     met:

       Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.

       Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.

       Neither name of the Omega Project for Statistical Computing nor the 
       names of its contributors may be used to endorse or promote products 
       derived from this software without specific prior written permission.

     THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND ANY
     EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
     SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
     SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
     PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
     INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
     STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
     THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
*/

